using System.Linq;
using System.Collections.Generic;
using Ato.CD.Inbound.Shared;
using Ato.CD.Inbound.Shared202102;
using VaTS;
using DataContracts;
using Ato.EN.IntegrationServices.CodeGenerationPTR;
using Ato.EN.IntegrationServices.CodeGenerationIDS;

namespace Ato.CD.Inbound.PTR202102
{
    internal class CrossFormValidatorIDS : ICrossFormValidator
    {
        private PTR2021 ParentDocument { get; }

        private IDS2021 ChildDocument { get; }

        private List<Context> Contexts { get; }

        internal CrossFormValidatorIDS(PTR2021 report, BusinessDocument childDocument)
        {
            ParentDocument = report;
            ChildDocument = (IDS2021)childDocument.ConsumedReport;
            Contexts = ChildDocument.GetContexts();           
        }

        public IEnumerable<ProcessMessageDocument> ValidateCrossFormRules()
        {
            var response = new List<ProcessMessageDocument>();

            VRATOGEN402009(response);
            VRATOGEN438000(response);
            VRATOGEN438001(response);
            VRATOPTR438082(response);

            return response;
        }

        #region VR.ATO.GEN.402009
        /* 
        Rule ID: VR.ATO.GEN.402009

        IF (RP:entity.identifier.TFN <> PARENT RETURN:RP:entity.identifier.TFN)
            RETURN VALIDATION MESSAGE
        ENDIF
        */
        private void VRATOGEN402009(List<ProcessMessageDocument> response)
        {
            bool assertion = !ParentDocument.RPIdentifierTFN.Equals(ChildDocument.RPIdentifierTFN);

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                    LongDescription = "Your supplied TFN does not match the TFN supplied on the parent form it was submitted with",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.402009" } }
                });
            }
        }
        #endregion

        #region VR.ATO.GEN.438000
        /* 
        Rule ID: VR.ATO.GEN.438000

        IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
            RETURN VALIDATION MESSAGE
        ENDIF
        */
        private void VRATOGEN438000(List<ProcessMessageDocument> response)
        {
            response.AddRange(
                from context in Contexts
                where context.StartDate.GetValueOrDefault() != ParentDocument.RPStartDate.GetValueOrDefault()
                select new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438000",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    Location = $"/xbrli:xbrl/xbrli:context[{context.Index}]/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                }
            );
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438001
        /* 
        Rule ID: VR.ATO.GEN.438001

        IF (period.EndDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.EndDate
            RETURN VALIDATION MESSAGE
        ENDIF
        */
        private void VRATOGEN438001(List<ProcessMessageDocument> response)
        {
            response.AddRange(
                from context in Contexts
                where context.EndDate.GetValueOrDefault() != ParentDocument.RPEndDate.GetValueOrDefault()
                select new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438001",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    Location = $"/xbrli:xbrl/xbrli:context[{context.Index}]/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.438001" } }
                }
            );
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.PTR.438082
        // IF [PTR355] = TRUE AND (COUNT(SCHEDULE = "IDS") = 1 AND [IDS425] = TRUE)
        //  RETURN VALIDATION MESSAGE
        // ENDIF
        // [IDS425] = IDS:RP:rvctc4.02.16:InternationalDealings.SmallBusinessConcessionsEligibility.Indicator
        // [PTR355] = PTR:RP:pyde.02.31:OrganisationDetails.SignificantGlobalEntityStatus.Indicator
        private void VRATOPTR438082(List<ProcessMessageDocument> response)
        {
            bool assertion = ParentDocument.PTR355.GetValueOrDefault() && ChildDocument.IDS425.GetValueOrDefault();
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.PTR.438082",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Significant Global Entity question on the International Dealings Schedule (IDS) form must not be 'TRUE'.",
                    LongDescription = "Your supplied TFN does not match the TFN supplied on the parent form it was submitted with",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.SignificantGlobalEntityStatus.Indicator",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.PTR.438082" } }
                });
            }
        }
        #endregion
    }
}