using System.Linq;
using System.Collections.Generic;
using Ato.CD.Inbound.Shared;
using Ato.CD.Inbound.Shared202102;
using Ato.EN.IntegrationServices.Contrib.VaTS;
using Ato.EN.IntegrationServices.Document.DataContracts;
using Ato.EN.IntegrationServices.CodeGenerationTRT;
using Ato.EN.IntegrationServices.CodeGenerationIDS;

namespace Ato.CD.Inbound.TRT202102
{
    internal class CrossFormValidatorIDS : ICrossFormValidator
    {
        private TRT2021 ParentDocument { get; }

        private IDS2021 ChildDocument { get; }

        private List<Context> Contexts { get; }

        internal CrossFormValidatorIDS(TRT2021 report, BusinessDocument childDocument)
        {
            ParentDocument = report;
            ChildDocument = (IDS2021)childDocument.ConsumedReport;
            Contexts = ChildDocument.GetContexts();
        }

        public IEnumerable<ProcessMessageDocument> ValidateCrossFormRules()
        {
            var response = new List<ProcessMessageDocument>();

            VRATOGEN402009(response);
            VRATOGEN438000(response);
            VRATOGEN438001(response);
            VRATOTRT432649(response);

            return response;
        }

        #region VR.ATO.GEN.402009

        /*  VR.ATO.GEN.402009
                    Your supplied TFN does not match the TFN supplied on the form it was submitted with

                Legacy Rule Format:
                    IF (RP:entity.identifier.TFN <> PARENT RETURN:RP:entity.identifier.TFN)
                    RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^TFN <> ^ParentTFN

        Data Elements:
            
                    ^ParentTFN = ParentTFN
            
                    ^TFN = RP
                    */
        private void VRATOGEN402009(List<ProcessMessageDocument> response )
        {
            bool assertion = ! ChildDocument.RPIdentifierTFN.Equals(ParentDocument.RPIdentifierTFN ) ;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                    LongDescription = @"Your supplied TFN does not match the TFN supplied on the parent form it was submitted with",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.402009" } }
                };
                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.402009

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        private void VRATOGEN438000(List<ProcessMessageDocument> response )
        {
            response.AddRange(from context in Contexts
                              where context.StartDate.GetValueOrDefault() != ParentDocument.RPStartDate.GetValueOrDefault()
                              select new ProcessMessageDocument
                              {
                                  Code = "CMN.ATO.GEN.438000",
                                  Severity = ProcessMessageSeverity.Error,
                                  Description = @"The context period start date is incorrect.",
                                  LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                                  Location = $"/xbrli:xbrl/xbrli:context[{context.Index}]/xbrli:period/xbrli:startDate",
                                  Parameters = new ProcessMessageParameters { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } }
                              });
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP.BranchOperations:RP.BranchOperations.{InternallyRecordedDealings}
                    */
        private void VRATOGEN438001(List<ProcessMessageDocument> response )
        { 
            response.AddRange(from context in Contexts
                              where context.EndDate.GetValueOrDefault() != ParentDocument.RPEndDate.GetValueOrDefault()
                              select new ProcessMessageDocument
                              {
                                  Code = "CMN.ATO.GEN.438001",
                                  Severity = ProcessMessageSeverity.Error,
                                  Description = @"The context period end date is incorrect.",
                                  LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                                  Location = $"/xbrli:xbrl/xbrli:context[{context.Index}]/xbrli:period/xbrli:endDate",
                                  Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.438001" } }
                              });
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.TRT.432649

        /*  VR.ATO.TRT.432649
                    If there is a yes response to question 4 label A on the IDS, there must also be a yes response to question 29 label C (transactions with specified countries) on the trust or partnership return form.

                Legacy Rule Format:
                    IF [TRT151] = FALSE AND [IDS9] = TRUE
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS9] = IDS:RP:rvctc4.02.05:InternationalDealings.SpecifiedCountriesTransactions.Indicator
                    [TRT151] = TRT:RP:rvctc4.02.11:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator

            Technical Business Rule Format:
                    ^TRT151 = FALSE AND ^IDS9 = TRUE

        Data Elements:
            
                    RP:^TRT151 = tns:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator
            
                    RP:^IDS9 = IDS:RP:rvctc4.02.05:InternationalDealings.SpecifiedCountriesTransactions.Indicator
                    */
        private void VRATOTRT432649(List<ProcessMessageDocument> response )
        {
            bool assertion =    ParentDocument.TRT151.GetValueOrDefault() == false && ChildDocument.IDS9.GetValueOrDefault() == true;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.432649",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Transactions with specified countries question incorrect",
                    LongDescription = @"If there is a 'True' response at 'Did you have dealings with international related parties located in specified countries?' on the IDS then the response at Overseas transactions - Transactions with specified countries on the trust return must be 'True'.",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432649" },
                        new ProcessMessageParameter { Name = "TRT151", Value = ParentDocument.TRT151.ToString () }
                    }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432649
    }
}