using System;
using System.Linq;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using Ato.CD.Inbound.Shared;
using Ato.CD.Inbound.Shared201802;
using Ato.EN.IntegrationServices.CodeGenerationCGTS;
using Ato.EN.IntegrationServices.CodeGenerationTRTAMI;
using VaTS;
using DataContracts;

namespace Ato.CD.Inbound.TRTAMI202102
{
    internal class CrossFormValidatorCGTS : ICrossFormValidator
    {
        private TRTAMI2021 ParentDocument { get; }
        private CGTS2018 ChildDocument { get; }
        private List<Context> Contexts { get; }

        internal CrossFormValidatorCGTS(TRTAMI2021 report, BusinessDocument childDocument)
        {
            ParentDocument = report;
            ChildDocument = (CGTS2018)childDocument.ConsumedReport;
            Contexts = ChildDocument.GetContexts();
        }

        public IEnumerable<ProcessMessageDocument> ValidateCrossFormRules()
        {
            var response = new List<ProcessMessageDocument>();

            VRATOGEN402009(response);
            VRATOGEN438000(response);
            VRATOGEN438001(response);            
            VRATOCGTS402171(response);
            VRATOCGTS402165(response);
            VRATOCGTS402174(response);
            VRATOCGTS402226(response);
            VRATOCGTS402240(response);
            VRATOTRTAMI000243(response);
            VRATOTRTAMI000245(response);

            return response;
        }

        #region VR.ATO.GEN.402009
        /* VR.ATO.GEN.402009
            IF (RP:entity.identifier.TFN <> PARENT RETURN:RP:entity.identifier.TFN)
                RETURN VALIDATION MESSAGE
            ENDIF 
        */
        private void VRATOGEN402009(List<ProcessMessageDocument> response)
        {
            bool assertion = !ParentDocument.RPIdentifierTFN.Equals(ChildDocument.RPIdentifierTFN);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                    LongDescription = @"Your supplied TFN does not match the TFN supplied on the parent form it was submitted with",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.402009" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.402009
        #region VR.ATO.GEN.438000
        /*  VR.ATO.GEN.438000
            IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF
        */
        private void VRATOGEN438000(List<ProcessMessageDocument> response)
        {
            response.AddRange(from context in Contexts
                              where context.StartDate.GetValueOrDefault() != ParentDocument.RPStartDate.GetValueOrDefault()
                              select new ProcessMessageDocument
                              {
                                  Code = "CMN.ATO.GEN.438000",
                                  Severity = ProcessMessageSeverity.Error,
                                  Description = @"The context period start date is incorrect.",
                                  LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                                  Location = $"/xbrli:xbrl/xbrli:context[{context.Index}]/xbrli:period/xbrli:startDate",
                                  Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.438000" } }
                              });
        }
        #endregion // VR.ATO.GEN.438000
        #region VR.ATO.GEN.438001
        /*  VR.ATO.GEN.438001
            IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF
        */
        private void VRATOGEN438001(List<ProcessMessageDocument> response)
        {
            response.AddRange(from context in Contexts
                              where context.EndDate.GetValueOrDefault() != ParentDocument.RPEndDate.GetValueOrDefault()
                              select new ProcessMessageDocument
                              {
                                  Code = "CMN.ATO.GEN.438001",
                                  Severity = ProcessMessageSeverity.Error,
                                  Description = @"The context period end date is incorrect.",
                                  LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                                  Location = $"/xbrli:xbrl/xbrli:context[{context.Index}]/xbrli:period/xbrli:endDate",
                                  Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.438001" } }
                              });
        }
        #endregion // VR.ATO.GEN.438001
        #region VR.ATO.CGTS.402165
        /* VR.ATO.CGTS.402165

             IF (PARENT RETURN <> "CTR") AND ([CGTS122] <> NULL)  
               RETURN VALIDATION MESSAGE
             ENDIF

            [CGTS122] = CGTS:RP.Applied:rvctc3.02.11:Capital.Losses.TransferredTotal.Amount
    
        */
        private void VRATOCGTS402165(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.CGTS122.HasValue;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.CGTS.402165",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total losses transferred in applied is only valid for a company",
                    Location = "/xbrli:xbrl/tns:Capital.Losses.TransferredTotal.Amount",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402165" } }
                };

                processMessage.Parameters.Add(new ProcessMessageParameter
                {
                    Name = "CGTS122",
                    Value = ChildDocument.CGTS122.Value.ToString()
                });

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CGTS.402165
        #region VR.ATO.CGTS.402171
        /* VR.ATO.CGTS.402171

            IF PARENT RETURN = "TRTAMI","FITR","TRT" AND([CGTS131] > 0) AND([CGTS125] > 0)
                RETURN VALIDATION MESSAGE
            ENDIF

            [CGTS125] = CGTS:RP.CGTOther.Unapplied:rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
            [CGTS131] = CGTS:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount        
    
        */
        private void VRATOCGTS402171(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.CGTS131.GetValueOrDefault() > 0 && ChildDocument.CGTS125.GetValueOrDefault() > 0;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.CGTS.402171",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"A capital loss must not be carried forward when a positive net capital gain amount has been provided",
                    Location = "/xbrli:xbrl/tns:Capital.Losses.CarriedForward.Net.Amount",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402171" } }
                };

                processMessage.Parameters.Add(new ProcessMessageParameter
                {
                    Name = "CGTS131",
                    Value = ChildDocument.CGTS131.GetValueOrDefault().ToString()
                });

                processMessage.Parameters.Add(new ProcessMessageParameter
                {
                    Name = "CGTS125",
                    Value = ChildDocument.CGTS125.GetValueOrDefault().ToString()
                });

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CGTS.402171
        #region VR.ATO.CGTS.402174
        /* VR.ATO.CGTS.402174

            IF PARENT RETURN = SET(""TRT"", ""CTR"",""TRTAMI"") AND  [CGTS126] > ((([CGTS118] - [CGTS123]) *0.5) +1)
               RETURN VALIDATION MESSAGE
            ENDIF

            [CGTS118] = CGTS:RP:rvctc3.02.11:Capital.Gains.Total.Amount
            [CGTS123] = CGTS:RP.Applied:rvctc3.02.11:Capital.Losses.Total.Amount
            [CGTS126] = CGTS:RP.Applied:gfagc.02.06:TaxConcession.CapitalGains.DiscountTotal.Amount  
    
        */
        private void VRATOCGTS402174(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.CGTS126.GetValueOrDefault() > (((ChildDocument.CGTS118.GetValueOrDefault() - ChildDocument.CGTS123.GetValueOrDefault()) * 0.5m) + 1);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.CGTS.402174",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The CGT discount rate applied to capital gains by a trust or company must not be greater than 50%",
                    Location = "/xbrli:xbrl/tns:TaxConcession.CapitalGains.DiscountTotal.Amount",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402174" } }
                };

                processMessage.Parameters.Add(new ProcessMessageParameter
                {
                    Name = "CGTS118",
                    Value = ChildDocument.CGTS118.GetValueOrDefault().ToString()
                });

                processMessage.Parameters.Add(new ProcessMessageParameter
                {
                    Name = "CGTS123",
                    Value = ChildDocument.CGTS123.GetValueOrDefault().ToString()
                });

                processMessage.Parameters.Add(new ProcessMessageParameter
                {
                    Name = "CGTS126",
                    Value = ChildDocument.CGTS126.GetValueOrDefault().ToString()
                });

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CGTS.402174
        #region VR.ATO.CGTS.402226
        /*  VR.ATO.CGTS.402226
          
            IF (PARENT RETURN:RP:pyin.xx.xx:Report.Amendment.Indicator = TRUE) AND ([CGTS150] <> NULL AND [CGTS151] <> NULL)
               RETURN VALIDATION MESSAGE
            ENDIF

            [CGTS150] = CGTS:RP:pyin.02.37:Report.TargetFinancial.Year
            [CGTS151] = CGTS:RP:bafot.02.26:Miscellaneous.RequestedAmendment.Amount
           
        */
        private void VRATOCGTS402226(List<ProcessMessageDocument> response)
        {
            bool assertion = ParentDocument.TRTAMI6.GetValueOrDefault() && (ChildDocument.CGTS150.HasValue && ChildDocument.CGTS151.HasValue);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.CGTS.402226",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Income year earnout right created and Amount to be amended cannot be supplied when Parent return Amendment indicator is present.",
                    Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402226" } }
                };

                processMessage.Parameters.Add(new ProcessMessageParameter
                {
                    Name = "CGTS150",
                    Value = ChildDocument.CGTS150.GetValueOrDefault().ToString()
                });

                processMessage.Parameters.Add(new ProcessMessageParameter
                {
                    Name = "CGTS151",
                    //may not be 100% correct, need to revisit during unit test dev. CGTS151 in wrong location in taxonomy
                    Value = ChildDocument.CGTS151.GetValueOrDefault().ToString()
                });

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CGTS.402226       
        #region VR.ATO.CGTS.402240
        /* VR.ATO.CGTS.402240

            IF([CGTS150] <> NULL) AND([CGTS150] = (PARENT RETURN:RP:pyin.xx.xx:Report.TargetFinancial.Year))
               RETURN VALIDATION MESSAGE
            ENDIF

            [CGTS150] = CGTS:RP:pyin.02.37:Report.TargetFinancial.Year         
        */
        private void VRATOCGTS402240(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.CGTS150.HasValue && ChildDocument.CGTS150 == ParentDocument.TRTAMI1.GetValueOrDefault();

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.CGTS.402240",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Income year earnout right created incorrect",
                    Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402240" } }
                };

                processMessage.Parameters.Add(new ProcessMessageParameter
                {
                    Name = "CGTS150",
                    Value = ChildDocument.CGTS150.GetValueOrDefault().ToString()
                });

                response.Add(processMessage);
        }
        }
        #endregion // VR.ATO.CGTS.402240
        #region VR.ATO.TRTAMI.000243
        /*  VR.ATO.TRTAMI.000243
            IF [TRTAMI102] = TRUE AND COUNT(SCHEDULE = "CGTS") > 0 AND ANY OCCURRENCE OF([CGTS153]) = NULLORBLANK
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    */
        private void VRATOTRTAMI000243(List<ProcessMessageDocument> response)
        {
            bool assertion = ParentDocument.TRTAMI102.GetValueOrDefault() && (string.IsNullOrWhiteSpace(ChildDocument.CGTS153));
            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
        {

                    Code = "CMN.ATO.TRTAMI.000243",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Name of AMIT Class must be provided for each CGT schedule",
                    LongDescription = @"When the response to 'Have you chosen to treat separate classes in the trust as a separate AMIT?' is 'True' (yes), the Name of AMIT Class must be provided for each Capital Gains Tax (CGT) schedule",
                    Location = $"/xbrli:xbrl/xbrli:context[]",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000243" } }
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                {
                    Name = "TRTAMI102",
                    Value = ParentDocument.TRTAMI102.GetValueOrDefault().ToString()
                });

                processMessage.Parameters.Add(new ProcessMessageParameter
                {
                    Name = "CGTS153",
                    //may not be 100% correct, need to revisit during unit test dev. CGTS151 in wrong location in taxonomy
                    Value = ChildDocument.CGTS153
                });
                response.Add(processMessage);

            }
        }
        #endregion // VR.ATO.TRTAMI.000243
        #region VR.ATO.TRTAMI.000245
        /*  VR.ATO.TRTAMI.000245
            IF [TRTAMI102] = FALSE AND COUNT(SCHEDULE = "CGTS") > 0 
            AND (ANY OCCURRENCE OF([CGTS154]) <> SET(NULL,0) OR ANY OCCURRENCE OF([CGTS155]) <> SET(NULL,0))
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    */
        private void VRATOTRTAMI000245(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            assertion = (ParentDocument.TRTAMI102.GetValueOrDefault() == false && (!IsMatch(Convert.ToInt32(ChildDocument.CGTS154), @"^(NULL|0)$", RegexOptions.IgnoreCase) || !IsMatch(Convert.ToInt32(ChildDocument.CGTS155), @"^(NULL|0)$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRTAMI.000245",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total capital gains/losses from transfers to other classes amounts must not be provided in CGT schedule",
                    LongDescription = @"When the response to 'Have you chosen to treat separate classes in the trust as a separate AMIT?' is 'False' (no), the Total capital gains/losses from transfers to other classes amounts for the Capital Gains Tax (CGT) schedule must not be provided",
                    Location = $"/xbrli:xbrl/xbrli:context[]",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000245" } }
                };
                response.Add(processMessage);

            }
        }
        #endregion // VR.ATO.TRTAMI.000245

        private static bool IsMatch(int? field, string expression, RegexOptions options = RegexOptions.None)
            => field != null && Regex.IsMatch(Convert.ToString(field.Value), expression, options);
    }
}